%% DG-System: was die Eigenwerte der Jacobi-Matrix aussagen
%

% In der Vorlesung vom 12. Mai 2022 wurde für die DG y'' + y' + y = 0 das
% entsprechende System 1. Ordnung aufgestellt, die Eigenwerte der
% Jacobi-Matrix berechnet und daraufhin die Lösung skizziert. (ab etwa
% 1:27) Hier wird numerisch nachgerechnet.

%% symbolische Lösung der DG 2. Ordnung
syms y(x)

ode = diff(y,x,2) + diff(y,x,1) + y(x) == 0;
dsolve(ode) 

% die allgemeine Lösung: exponentiell abklingende Sinus- und
% Cosinusfunktionen. 


%% das umgeformte System 1. Ordnung
% Jacobi-Matrix 
J = [ 0 1; -1 -1];

% damit lässt sich das DG-System sehr einfach definieren. (Das funktioniert
% bei linearen Systemen mit konstannten Koeffizienten immer so!)
f = @(t,z) J*z;

%% Eigenwerte von J
eig(J)

% Bedeutung: Eigenwerte delta +/- i omega entsprechen Lösungen  
%   exp(delta t)*sin(omega t), exp(delta t)*cos(omega t). 
% Vergleiche mit symbolischer Lösung von oben!

%% Numerische Lösung mit ode45
% hängt noch von Anfangsbedingungen ab. Die sind hier so gewählt, dass sich
% annähernd dieselbe Lösung wie auf der Tafelskizze ergibt.
sol=ode45(f,[0 10],[1,-1/2]);

% An der Tafel ist die waagrechte Achse einmal mit x, einmal mit t
% bezeichnet. Abgesehen davon gleicht die numerische Lösung der Tafelskizze
% recht gut.

tt = linspace(0,7);
yy = deval(sol,tt,1);

% Plot der Lösung und der Abklingkurven
plot(tt,yy,tt,exp(-tt/2),'.c',tt,-exp(-tt/2),'.c')
grid on

%% Numerische Lösung mit explizitem Eulerverfahren
% definiere Schrittweite, Lösungsvektoren 
h = 0.125;
t = 0:h:10;
z = zeros(2,length(t));

% Anfangsbedingungen
z(1,1)=1; z(2,1)=-1/2;

for i=1:length(t)-1
    z(:,i+1) = z(:,i) + h*J*z(:,i);
end
% den Rechenschritt hätte man auch so schreiben können:
%    z(:,i+1) = (eye(2) + h*J) *z(:,i);


% plotte numerische Lösung des expliziten Eulerverfahrens und genaue Lösung
tt = linspace(0,10);
yy = deval(sol,tt,1);
plot(t,z(1,:),'o:',tt,yy)



